/* xpyre/xmms.c: Interface to xmms.
 *
 * $Id: xmms.c,v 1.4 2002/12/08 02:26:08 andy Exp $
 *
 * Copyright (C) 2002 Andy Goth <unununium@openverse.com>
 * For more information visit http://ioioio.net/devel/xpyre/
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA  02111-1307, USA. */

#include <allegro.h>
#include <pthread.h>
#include <string.h>
#include <xmms/configfile.h>
#include <xmms/plugin.h>
#include <xmms/xmmsctrl.h>
#include "font.h"
#include "xpyre.h"

/* Prototypes. */
static void init         (void);
static void cleanup      (void);
static void render_pcm   (gint16 data[2][512]);
static void load_config  (void);

static void nop(void) {}

static VisPlugin xpyre;

static int called_by_init;

/* The current pcm audio data. */
pthread_mutex_t pcm_data_mutex;
gint16 pcm_data[512];

/* Common configuration. */
config_t config;

/* Current track. */
track_t track;

/* Used by xmms to get xpyre's plugin information. */
VisPlugin* get_vplugin_info(void) {return &xpyre;}

/* Interfaces to standard xmms functions. */
void xmms_disable(void) {xpyre.disable_plugin(&xpyre);                 }
void xmms_prior  (void) {xmms_remote_playlist_prev(xpyre.xmms_session);}
void xmms_play   (void) {xmms_remote_play(xpyre.xmms_session);         }
void xmms_pause  (void) {xmms_remote_pause(xpyre.xmms_session);        }
void xmms_stop   (void) {xmms_remote_stop(xpyre.xmms_session);         }
void xmms_next   (void) {xmms_remote_playlist_next(xpyre.xmms_session);}

/* Gets track information. */
int xmms_track_info(void)
{
    /* Get the track information. */
    int num = xmms_remote_get_playlist_pos(xpyre.xmms_session);
    char* title = xmms_remote_get_playlist_title(xpyre.xmms_session, num);

    /* TODO: track length.  Hmm, how do I deal with broken tracks whose length
     * changes constantly?  What about tracks with no length? */

    if (title == NULL) {
        /* The playlist is empty. */

        title = "(empty playlist)";
        num = -1;
    }

    if (num != track.num || strcmp(title, track.title) != 0) {
        /* Track number or title changed. */

        int title_len = strlen(title);
        if (title_len >= track.title_cap) {
            track.title = realloc(track.title, title_len + 1);
            track.title_cap = title_len + 1;
        }
        memcpy(track.title, title, title_len);
        track.title[title_len] = 0;

        track.num = num;

        return 1;
    } else {
        /* Same old song. */

        return 0;
    }
}

/* This is passed to xmms to register the plugin. */
static VisPlugin xpyre = {
    /* The following fields are supplied by xmms.  They tell xpyre how to
     * interface with xmms. */
.handle               = NULL,
.filename             = NULL,
.xmms_session         = 0,
.disable_plugin       = NULL,

    /* Parameters describing the plugin to xmms. */
.description          = "xpyre",
.num_pcm_chs_wanted   = 1,
.num_freq_chs_wanted  = 0,

    /* These functions tell xmms how to interface with xpyre. */
.init                 = init,
.cleanup              = cleanup,
.render_pcm           = render_pcm,

    /* xpyre doesn't implement these functions. */
.about                = NULL,
.configure            = NULL,
.playback_start       = NULL,
.playback_stop        = NULL,
.render_freq          = NULL
};

/* Used by xmms to activate xpyre. */
static void init(void)
{
    called_by_init = 0;
    pthread_mutex_init(&pcm_data_mutex, NULL);

    /* Initialize random number generator. */
    srand(time(NULL));
    my_rand(0);

    /* Fetch settings. */
    load_config();

    /* Get the current track. */
    track = (track_t){-1, NULL, 0, -1};
    track.title = strdup("(playlist not loaded)");
    track.title_cap = strlen(track.title);

    /* Start Allegro. */
    install_allegro(SYSTEM_AUTODETECT, &errno, NULL);
    install_keyboard();
    font = &my_font_pcx;
    allegro_404_char = 127;
    text_mode(-1);

    /* Allegro window settings. */
    set_window_title("xpyre");
    set_window_close_hook(nop);

    /* Initialize the drawing code. */
    if (!draw_start()) {
        called_by_init = 1;
        allegro_exit();
        xmms_disable();
        return;
    }

    /* Start the timer. */
    timer_start(1e6 / config.fps);

    /* Start the main thread. */
    main_start();
}

/* Used by xmms to deactivate xpyre. */
static void cleanup(void)
{
    if (called_by_init) return;

    main_stop();
    timer_stop();
    draw_stop();
    allegro_exit();

    if (track.title != NULL) {
        free(track.title);
        track.title = NULL;
        track.title_cap = 0;
    }

    pthread_mutex_destroy(&pcm_data_mutex);
}

/* Used by xmms to send pcm data to xpyre. */
static void render_pcm(gint16 data[2][512])
{
    if (pthread_mutex_trylock(&pcm_data_mutex) != EBUSY) {
        memcpy(pcm_data, data[0], sizeof(gint16) * 512);
        pthread_mutex_unlock(&pcm_data_mutex);
    }
}

/* Fills config with user settings. */
static void load_config(void)
{
    ConfigFile* f;
    gchar* buf;
    
    /* Defaults. */
    config = (config_t){
    .width        = 320,
    .height       = 240,
    .fullscreen   = FALSE,
    .fps          = 60,
    .draw_title   = TRUE,
    .bg_color     = 0,
    .draw_top     = TRUE,
    .random_top   = TRUE,
    .draw_fill    = TRUE,
    .random_fill  = TRUE
    };
    
    buf = malloc(4097);
    if (buf == NULL) return;

    /* Open the configuration file. */
    fix_filename_path(buf, "~/.xmms/xpyre/config", 4097);
    f = xmms_cfg_open_file(buf);
    free(buf); buf = NULL;
    if (f == NULL) return;

    /* Read all known fields. */
    xmms_cfg_read_int(f, "xpyre", "width",  &config.width);
    xmms_cfg_read_int(f, "xpyre", "height", &config.height);
    xmms_cfg_read_int(f, "xpyre", "fps",    &config.fps);

    xmms_cfg_read_boolean(f, "xpyre", "draw_title", &config.draw_title);
    xmms_cfg_read_string (f, "xpyre", "bg_color",   &buf);

    if (buf != NULL) {
        config.bg_color = rrggbb_to_color(buf, makecol32(0, 0, 0));
        free(buf);
    }

    xmms_cfg_read_boolean(f, "xpyre", "draw_top",    &config.draw_top);
    xmms_cfg_read_boolean(f, "xpyre", "random_top",  &config.random_top);

    xmms_cfg_read_boolean(f, "xpyre", "draw_fill",   &config.draw_fill);
    xmms_cfg_read_boolean(f, "xpyre", "random_fill", &config.random_fill);

    /* Close the configuration file. */
    xmms_cfg_free(f);
}

/* vim: set ts=4 sts=4 sw=4 tw=80 et: */
/* EOF */
